package de.hhu.ba.yoshikoWrapper.core;
//TODO: ADD LOGGER SYSTEM

import org.cytoscape.application.CyApplicationManager;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;

import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_std__vectorT_int_t;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_yskInput__LibraryInput;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_ysk__ClusterEditingSolutions;

public class Core {
	
	private static CyApplicationManager cy;
	

	public static void performYoshiko(int timeLimit) {
		CyNetwork currentNetwork = cy.getCurrentNetwork();
		if (currentNetwork == null) {
			//TODO
			return;
		}
		
		LibraryInterface.setTimeLimit(timeLimit);
		
		NodeMap nodeMap  = new NodeMap(currentNetwork);

		SWIGTYPE_p_yskInput__LibraryInput input = NetworkParser.parseNetwork(currentNetwork,nodeMap);
		SWIGTYPE_p_ysk__ClusterEditingSolutions solutions = LibraryInterface.processLibraryInput(input);
		long numberOfSolutions = LibraryInterface.ClusterEditingSolutions_getNumberOfSolutions(solutions);
		System.out.println("Found: "+numberOfSolutions+" solutions!");
		double modificationCost = LibraryInterface.ClusterEditingSolutions_getTotalCost(solutions);
		System.out.println("Paid a total modification cost of: "+modificationCost);
		for (long i=0;i<numberOfSolutions;i++) {
			System.out.println("Processing solution "+(i+1)+" of "+numberOfSolutions);
			String columnName = "YOSHIKO_SOLUTION_"+(i+1);
			currentNetwork.getDefaultNodeTable().createColumn(columnName, String.class, false);
			long numberOfClusters = LibraryInterface.ClusterEditingSolutions_getNumberOfClusters(solutions, i);
			for (long k=0;k<numberOfClusters;k++) {
				System.out.println("Processing cluster "+(k+1)+" of "+numberOfClusters);
				SWIGTYPE_p_std__vectorT_int_t cluster = LibraryInterface.ClusterEditingSolutions_getCluster(solutions, i, k);
				long sizeOfCluster = LibraryInterface.IntVector_size(cluster);
				for (int l=0;l<sizeOfCluster;l++) { //Unsafe mismatch int long
					System.out.println("Processing entry "+(l+1)+ " of "+sizeOfCluster);
					int nodeID = LibraryInterface.IntVector_get(cluster, l);
					CyNode node = nodeMap.indexOf(nodeID); //<<< Another int/long conversion
					if (node == null) {
						System.out.println("FATAL ERROR: Node was not previously mapped!");
						return;
					}
					currentNetwork.getRow(node).set(columnName, ""+(k+1));
				}
			}
		}
		
		LibraryInterface.delete_LibraryInput(input);		
	}
	
	
	//SETTER / GETTER METHODS
	
	public static void setApplicationManager(CyApplicationManager cyApplicationManager) {
		cy = cyApplicationManager;
	}

	public static CyApplicationManager getApplicationManager() {
		return cy;
	}

}
