package de.hhu.ba.yoshikoWrapper.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import de.hhu.ba.yoshikoWrapper.core.Core;
import de.hhu.ba.yoshikoWrapper.core.YoshikoLoader;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;

/**This class describes the Swing Panel that the user interacts with in cytoscape
 * @author Philipp Spohr, Aug 6, 2017
 *
 */
@SuppressWarnings("serial")
public class MainPanel extends ComfortPanel implements CytoPanelComponent {

	//SYMBOLIC LINKS
	private MainPanel self = this; //for lambda function references
	
	//SWING COMPONENTS
	private LibStatusPanel libStatusPanel;
	private JButton searchLibButton;
	private JLabel yoshikoVersionLabel;

	private EditCostPanel ecPanel;
	
	private ButtonGroup heuristicGroup;
	private JRadioButton useHeuristic;
	private JRadioButton useILP;
	
	private TimeLimitSetter timeLimitSetter;
	
	private ReductionRulesChooser reductionRulesChooser;
	
	private JCheckBox useTriangleCutsBox;
	private JCheckBox usePartitionCutsBox;
	


	/**
	 * Main constructor, creates a new Panel and initializes subcomponents
	 */
	public MainPanel() {
		
		this.setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
		
		//SWING COMPONENT INITIALIZATION
		libStatusPanel = new LibStatusPanel();
		libStatusPanel.setStyle(YoshikoLoader.isLibraryLoaded());
		
		yoshikoVersionLabel = new JLabel("YOSHIKO VERSION");
		if (LibraryInterface.getVersionString() != null) {
			yoshikoVersionLabel.setText(LibraryInterface.getVersionString());
		}
		searchLibButton = new JButton("POINT TO LIB");
		
		ecPanel = new EditCostPanel();
		
		heuristicGroup = new ButtonGroup();
		useILP = new JRadioButton("Use Integer Linear Programming");
		useILP.setSelected(true);
		useHeuristic = new JRadioButton("Use Heuristic");
		heuristicGroup.add(useILP);
		heuristicGroup.add(useHeuristic);
		
		searchLibButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				final YLibChooser c = new YLibChooser();
				int returnVal = c.showOpenDialog(self);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					YoshikoLoader.loadLibrary(c.getSelectedFile().getAbsolutePath());
				}
				libStatusPanel.setStyle(YoshikoLoader.isLibraryLoaded());
				yoshikoVersionLabel.setText(LibraryInterface.getVersionString());
			}
			
		});

		JButton runButton = new JButton("RUN");
		runButton.addActionListener(new ActionListener() {


			@Override
			public void actionPerformed(ActionEvent e) {
				if (YoshikoLoader.isLibraryLoaded()){
					Core.performYoshiko(timeLimitSetter.getTimeLimit());

				}
			}
			
		});

		reductionRulesChooser = new ReductionRulesChooser();
		
		timeLimitSetter = new TimeLimitSetter();
		
		useTriangleCutsBox = new JCheckBox("Use Triangle Cuts");
		usePartitionCutsBox = new JCheckBox("Use Partition Cuts");

		
		this.addAll(
				libStatusPanel,
				yoshikoVersionLabel,
				searchLibButton,
				ecPanel,
				useILP,
				useHeuristic,
				timeLimitSetter,
				reductionRulesChooser,
				useTriangleCutsBox,
				usePartitionCutsBox,
				runButton
				);
		
		this.setVisible(true);
	}

	
	
	//GETTER / SETTER

	public Component getComponent() {
		return this;
	}


	/* (non-Javadoc)
	 * @see org.cytoscape.application.swing.CytoPanelComponent#getCytoPanelName()
	 */
	public CytoPanelName getCytoPanelName() {
		//By convention most plugins that provide a "toolbox"-like interface use the WEST orientation
		return CytoPanelName.WEST;
	}

	public String getTitle() {
		//TODO: Be creative I guess
		return "Yoshiko Wrapper Panel";
	}


	public Icon getIcon() {
		//TODO: 
		return null;
	}
}
